"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const js_sha512_1 = require("js-sha512");
const path_1 = require("path");
const auth_provider_1 = require("./auth-provider");
const crypto = require("crypto");
class JsonProvider extends auth_provider_1.default {
    constructor(config, fs) {
        super();
        this.dbLocation = path_1.join(config.get("workDir"), "db");
        this.settings = config.get("auth");
        this.fs = fs;
    }
    userLogin(username, password) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = this.users.get(username);
            if (user && user.password === js_sha512_1.sha512(password)) {
                const token = yield JsonProvider.generateToken();
                this.tokens.set(token, username);
                yield this.updateTokenDB();
                return token;
            }
            throw Error("Error");
        });
    }
    userAdd(username, password, email) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.settings.register === true) {
                if (this.users.has(username)) {
                    throw Error("Forbidden");
                }
                this.users.set(username, {
                    username,
                    password: js_sha512_1.sha512(password),
                    email: email
                });
                yield this.updateUserDB();
                const token = yield JsonProvider.generateToken();
                this.tokens.set(token, username);
                yield this.updateTokenDB();
                return token;
            }
            throw Error("Error");
        });
    }
    userRemove(username, password) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.settings.remove === true) {
                const user = this.users.get(username);
                if (typeof user === "object" && user.password === js_sha512_1.sha512(password)) {
                    this.users.delete(username);
                    yield this.updateUserDB();
                    return true;
                }
            }
            return false;
        });
    }
    userLogout(token) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.tokens.has(token)) {
                this.tokens.delete(token);
            }
            yield this.updateTokenDB();
            return true;
        });
    }
    updateUserDB() {
        return __awaiter(this, void 0, void 0, function* () {
            const user_db_path = path_1.join(this.dbLocation, "user_db.json");
            const object = yield this.mapToObject(this.users);
            yield this.fs.writeFile(user_db_path, JSON.stringify(object), {
                mode: "0777"
            });
        });
    }
    updateTokenDB() {
        return __awaiter(this, void 0, void 0, function* () {
            const tokenLocation = path_1.join(this.dbLocation, "user_tokens.json");
            yield this.fs.writeFile(tokenLocation, JSON.stringify(yield this.mapToObject(this.tokens)), { mode: "0777" });
        });
    }
    initTokenDB() {
        return __awaiter(this, void 0, void 0, function* () {
            const user_token_path = path_1.join(this.dbLocation, "user_tokens.json");
            try {
                this.tokens = yield this.objToMap(JSON.parse(yield this.fs.readFile(user_token_path)));
            }
            catch (e) {
                this.tokens = new Map();
            }
        });
    }
    initUserDB() {
        return __awaiter(this, void 0, void 0, function* () {
            const user_db_path = path_1.join(this.dbLocation, "user_db.json");
            try {
                const userDb = yield this.fs.readFile(user_db_path);
                this.users = yield this.objToMap(JSON.parse(userDb));
            }
            catch (err) {
                yield this.fs.writeFile(user_db_path, JSON.stringify({}));
                this.users = new Map();
            }
        });
    }
    verifyToken(token) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.tokens.has(token)) {
                const tkn = this.tokens.get(token);
                if (typeof tkn === "string") {
                    return tkn;
                }
            }
            throw new Error("Invalid token");
        });
    }
    mapToObject(map) {
        return __awaiter(this, void 0, void 0, function* () {
            const object = {};
            map.forEach((v, k) => {
                object[k] = v;
            });
            return object;
        });
    }
    objToMap(obj) {
        return __awaiter(this, void 0, void 0, function* () {
            const map = new Map();
            yield Promise.all(Object.keys(obj).map((k) => __awaiter(this, void 0, void 0, function* () {
                map.set(k, obj[k]);
            })));
            return map;
        });
    }
    storageInit() {
        return __awaiter(this, void 0, void 0, function* () {
            const exists = yield this.fs.exists(this.dbLocation);
            if (!exists) {
                try {
                    yield this.fs.createDirectory(this.dbLocation);
                }
                catch (err) {
                    console.log(err);
                    return false;
                }
            }
            yield Promise.all([this.initUserDB(), this.initTokenDB()]);
            return true;
        });
    }
    static generateToken() {
        return __awaiter(this, void 0, void 0, function* () {
            return crypto.randomBytes(64).toString("hex");
        });
    }
}
exports.default = JsonProvider;
//# sourceMappingURL=json-provider.js.map